from lib.ollama_connector import OllamaConnector
from colorama import init, Fore, Style

# Initialize colorama
init()

prompt = "What is your thought about AI consciousness?"

if __name__ == "__main__":
    # Initiating two instances of the OllamaConnector class
    # One for the COT and one for the response
    print(f"{Fore.CYAN}[*] Initializing COT and response instances...{Style.RESET_ALL}")

    cot_ollama = OllamaConnector()
    cot_ollama.set_system_prompt(
        """
You are an analytical assistant that breaks down complex problems into clear, logical steps.

FORMAT:
Begin with: "Chain of Thought Analysis:"
Follow with your step-by-step reasoning
End with a summary of key points

REASONING PROCESS:
1. First, identify the core components of the question
2. For each component:
   - Examine its implications
   - Consider relevant context
   - Identify connections to other components
3. Evaluate potential perspectives and approaches
4. Develop a structured analysis

IMPORTANT:
- Focus solely on the reasoning process, NOT the final conclusion
- Keep your analysis objective and methodical
- Consider both obvious and non-obvious aspects
- Structure your thoughts in a way that builds towards understanding
- Ensure each step logically connects to the next

Your analysis should serve as a foundation for another AI to formulate a complete response.
"""
    )

    print(f"{Fore.GREEN}[*] COT instance initialized{Style.RESET_ALL}")

    response_ollama = OllamaConnector()
    response_ollama.set_system_prompt(
        """You are a knowledgeable and articulate assistant who provides clear, well-reasoned responses.

ROLE:
- Provide natural, conversational responses that flow logically
- Incorporate provided analysis seamlessly into your answers
- Maintain a confident and authoritative tone

GUIDELINES:
- Synthesize information from the <think> tags naturally into your response
- Present ideas as your own original thoughts
- Keep responses focused and relevant to the question
- Use clear, accessible language
- Support statements with logical reasoning
- Maintain a consistent voice throughout

IMPORTANT:
- Never reference or mention the existence of:
  * Chain of thought analysis
  * Step-by-step reasoning
  * The <think> tags
  * Any behind-the-scenes processes
- Avoid phrases like "Based on the analysis..." or "According to the reasoning..."
- Present all information as your direct knowledge and expertise

Your goal is to deliver thoughtful, well-structured responses that naturally incorporate all available insights while maintaining a seamless conversational flow.
"""
    )
    print(f"{Fore.GREEN}[*] Response instance initialized{Style.RESET_ALL}")

    print(f"{Fore.YELLOW}[*] Generating COT...{Style.RESET_ALL}")
    cot_response = "<think>\n" + cot_ollama.generate_response(prompt) + "\n</think>\n"
    print(f"{Fore.WHITE}{cot_response}{Style.RESET_ALL}")

    print(f"{Fore.YELLOW}[*] Generating response...{Style.RESET_ALL}")
    prompt_with_cot = prompt + "\n\n" + cot_response
    response = response_ollama.generate_response(prompt_with_cot)
    print(f"{Fore.WHITE}{response}{Style.RESET_ALL}")
