from typing import Generator
import ollama
import dotenv
import os

dotenv.load_dotenv()


class OllamaConnector:

    system_prompt = os.getenv("SYSTEM_PROMPT", "none")
    context = []
    model = os.getenv("MODEL", "llama3.1")

    def __init__(self):
        # Initialize the ollama client
        self.ollama = ollama.Client(
            host=os.getenv("OLLAMA_HOST", "localhost")
            + ":"
            + os.getenv("OLLAMA_PORT", "11434"),
        )
        # Defaulting to a system prompt if not set
        if self.system_prompt == "none":
            self.system_prompt = "You are a helpful assistant."

        # Insert the system prompt into the context
        self.context.append({"role": "system", "content": self.system_prompt})

    def generate_response(self, prompt: str) -> str:
        # Insert the prompt into the context
        self.context.append({"role": "user", "content": prompt})
        # Generate the response
        response = self.ollama.chat(
            model=self.model, messages=self.context, stream=False
        )
        return response["message"]["content"]

    def stream_response(self, prompt: str, print_response: bool = True) -> str:
        # Insert the prompt into the context
        self.context.append({"role": "user", "content": prompt})
        # Generate the response
        full_response = ""
        stream_response = self.ollama.chat(
            model=self.model, messages=self.context, stream=True
        )
        for chunk in stream_response:
            full_response += chunk.message.content
            if print_response:
                print(chunk.message.content, end="", flush=True)
        return full_response

    def reset_context(self):
        self.context = []

    def set_system_prompt(self, prompt: str):
        # Sanitize the prompt
        prompt = prompt.strip()
        prompt = prompt.replace("\n", " ")
        # Add the system prompt to the context
        self.system_prompt = prompt
        # Delete the old system prompt from the context
        self.context = [
            message for message in self.context if message["role"] != "system"
        ]
        # Insert the new system prompt into the context
        self.context.append({"role": "system", "content": self.system_prompt})

    def set_model(self, model: str):
        self.model = model

    def set_context(self, context: list):
        self.context = context

    def get_context(self):
        return self.context
